#ifndef __SHT11_H
#define __SHT11_H

/***********************************************************************************
Project: SHT11 program functions
Filename: SHT11.h
***********************************************************************************/

#include <math.h>
#include <stdlib.h>

unsigned char error, checksum,temp1, temp2;
unsigned char humi_val, humi_val1, temp_val;


//----------------------------------------------------------------------------------
// modul-var
//----------------------------------------------------------------------------------
enum {TEMP,HUMI};

#define	TRIS_DATA	TRISCbits.TRISC1
#define	TRIS_SCK	TRISCbits.TRISC0
#define	DATA	PORTCbits.RC1
#define SCK		PORTCbits.RC0
#define noACK	0
#define ACK		1

//adr command r/w
#define	STATUS_REG_W	0x06	//000 0011 0
#define STATUS_REG_R	0x07	//000 0011 1
#define MEASURE_TEMP	0x03	//000 0001 1
#define MEASURE_HUMI	0x05	//000 0010 1
#define	RESET			0x1e	//000 1111 0


//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge
//----------------------------------------------------------------------------------
unsigned char	s_write_byte(unsigned char value)

{
	unsigned char i, error=0;
	TRIS_DATA=0;	//make DATA-line output
	SCK=0;			//initial state
	
	for	( i=0x80 ; i>0 ; i/=2 ) //shift bit for masking
	{
		if ( i & value )
			DATA=1; 		//masking value with i , write to SENSI-BUS
		else
			DATA=0;
			
		SCK=1;	//clk for SENSI-BUS
		Nop();	//pulsewidth approx. 2 cycles
		SCK=0;
	}
	
//	DATA=0;			//release DATA-line
	TRIS_DATA=1;	//make DATA-line input
	Nop();

	SCK=1;		//clk #9 for ack
	error=DATA; //check ack (DATA will be pulled down by SHT11)
	SCK=0;

	return error; //error=1 in case of no acknowledge
}




//----------------------------------------------------------------------------------
// reads a byte from the Sensibus and gives an acknowledge in case of "ack=1"
//----------------------------------------------------------------------------------
unsigned char	s_read_byte(unsigned char ack)
{
	unsigned char i, val=0;
	
//	DATA=1;			//release DATA-line
	TRIS_DATA=1;	//make DATA-line input
	
	for ( i=0x80 ; i>0 ; i/=2 ) //shift bit for masking
	{
		SCK=1;	//clk for SENSI-BUS
		if (DATA)
			val=(val | i); //read bit
		SCK=0;
	}
	
	TRIS_DATA=0;	//make DATA-line output
	DATA = !ack;	//in case of "ack==1" pull down DATA-Line

	SCK=1; 			//clk #9 for ack
	Nop();			//pulsewidth approx. 2 cycles
	SCK=0;

//	DATA=1;			//release DATA-line
	TRIS_DATA=1;	//make DATA-line input
	
	return val;	
}



//----------------------------------------------------------------------------------
// generates a transmission start
//----------------------------------------------------------------------------------

void	s_transstart(void)
{
	TRIS_DATA=0;	//make DATA-line output

	DATA=1;
	SCK=0; //Initial state

	SCK=1;
	Nop();
	DATA=0;
	Nop();
	SCK=0;
	Nop();
	SCK=1;
	Nop();
	DATA=1;
	Nop();
	SCK=0;
}


//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
//----------------------------------------------------------------------------------

void	s_connectionreset(void)
{
	unsigned char i;
	TRIS_DATA=0;	//make DATA-line output
	
	DATA=1;
	SCK=0;	//Initial state
	
	for(i=0;i<12;i++)	//12 SCK cycles
	{
		SCK=1;
		Nop();
		SCK=0;
	}
	
	s_transstart();	//transmission start
}


//----------------------------------------------------------------------------------
// resets the sensor by a softreset
//----------------------------------------------------------------------------------

char	s_softreset(void)
{
	unsigned char error=0;
	
	s_connectionreset();	//reset communication
	
	error += s_write_byte(RESET);	//send RESET-command to sensor
	
	return error;	//error=1 in case of no response form the sensor
}



//----------------------------------------------------------------------------------
// reads the status register with checksum (8-bit)
//----------------------------------------------------------------------------------

char s_read_statusreg(char *p_value, char *p_checksum)
{
	char error=0;
	
	s_transstart(); //transmission start

	error = s_write_byte(STATUS_REG_R); //send command to sensor

	*p_value = s_read_byte(ACK);	//read status register (8-bit)
	*p_checksum = s_read_byte(noACK);	//read checksum (8-bit)

	return error;	//error=1 in case of no response form the sensor
}


//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
//----------------------------------------------------------------------------------

unsigned char s_write_statusreg(unsigned char p_value)
{
	char error=0;

	s_transstart(); //transmission start

	error+=s_write_byte(STATUS_REG_W);	//send command to sensor
	error+=s_write_byte(p_value); 		//send value of status register

//	WriteUSART(error);	//TESTE TESTE TESTE TESTE TESTE TESTE TESTE TESTE TESTE TESTE TESTE TESTE
	
	return error;	 //error>=1 in case of no response from the sensor
}


/*/----------------------------------------------------------------------------------
// makes a measurement (humidity/temperature) with checksum
//----------------------------------------------------------------------------------

void s_measure(unsigned char mode)
{
	
		s_transstart(); //transmission start
		
		switch	(mode)
		{				//send command to sensor
			case TEMP	:	s_write_byte(MEASURE_TEMP); break;
			case HUMI	:	s_write_byte(MEASURE_HUMI); break;
			default		:	break;
		}
	
		while (DATA);	//wait until SHT11 ends measurements
		
		WriteUSART(s_read_byte(ACK)); //read the first byte (MSB)
		WriteUSART(s_read_byte(noACK)); //read the second byte (LSB)

	return;// humi_val, humi_val1;
}
*/


/*/----------------------------------------------------------------------------------------
// calculates temperature [_C] and humidity [%RH]
// input : humi [Ticks] (12 bit)
// temp [Ticks] (14 bit)
// output: humi [%RH]
// temp [_C]
//----------------------------------------------------------------------------------------

void calc_sth11(float *p_humidity ,float *p_temperature)
{
	const float C1=-4.0; // for 12 Bit
	const float C2= 0.0405; // for 12 Bit
	const float C3=-0.0000028; // for 12 Bit
	const float T1=0.01; // for 14 Bit @ 5V
	const float T2=0.00008; // for 14 Bit @ 5V

	float rh=*p_humidity; // rh: Humidity [Ticks] 12 Bit
	float t=*p_temperature; // t: Temperature [Ticks] 14 Bit
	float rh_lin; // rh_lin: Humidity linear
	float rh_true; // rh_true: Temperature compensated humidity
	float t_C; // t_C : Temperature [_C]

	t_C=t*0.01-40; //calc. Temperature from ticks to [_C]
	rh_lin=C3*rh*rh + C2*rh + C1; //calc. Humidity from ticks to [%RH]
	rh_true=(t_C-25)*(T1+T2*rh)+rh_lin; //calc. Temperature compensated humidity [%RH]
	if(rh_true>100)
		rh_true=100; //cut if the value is outside of
	if(rh_true<0.1)
		rh_true=0.1; //the physical possible range

	*p_temperature=t_C; //return temperature [_C]
	*p_humidity=rh_true; //return humidity[%RH]
}



//--------------------------------------------------------------------
// calculates dew point
// input: humidity [%RH], temperature [_C]
// output: dew point [_C]
//--------------------------------------------------------------------

float calc_dewpoint(float h,float t)
{
	float logEx,dew_point ;
	logEx=0.66077+7.5*t/(237.3+t)+(log10(h)-2) ;
	dew_point = (logEx - 0.66077)*237.3/(0.66077+7.5-logEx) ;
	return dew_point;
}


//----------------------------------------------------------------------------------
// sample program that shows how to use SHT11 functions
// 1. connection reset
// 2. measure humidity [ticks](12 bit) and temperature [ticks](14 bit)
// 3. calculate humidity [%RH] and temperature [_C]
// 4. calculate dew point [_C]
// 5. print temperature, humidity, dew point
//----------------------------------------------------------------------------------

void main()
{
	union value humi_val,temp_val;
	float dew_point;
	unsigned char error,checksum;
	unsigned int i;
	
//	init_uart();
	s_connectionreset();
	
	while(1)
	{
		error=0;
		error+=s_measure((unsigned char*) &humi_val.i,&checksum,HUMI); //measure humidity
		error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP); //measure temperature
		if(error!=0) s_connectionreset(); //in case of an error: connection reset
		else
		{
			humi_val.f=(float)humi_val.i; //converts integer to float
			temp_val.f=(float)temp_val.i; //converts integer to float
			calc_sth11(&humi_val.f,&temp_val.f); //calculate humidity, temperature
//			dew_point=calc_dewpoint(humi_val.f,temp_val.f); //calculate dew point
//			printf("temp:%5.1fC humi:%5.1f%% dew point:%5.1fC\n",temp_val.f,humi_val.f,dew_point);
		}
//----------wait approx. 0.8s to avoid heating up SHTxx------------------------------
		for (i=0;i<40000;i++); //(be sure that the compiler doesn't eliminate this line!)
//-----------------------------------------------------------------------------------
	}
} 



*/


#endif
